@@ -109,9 +109,17 @@ class _Runner(object):
         self.original_callable = original_callable
         self.accepted_args = None
         self.call_count = 0
+        self.max_calls = None
 
     def run(self, *args, **kwargs):
         self.call_count += 1
+        if self.max_calls and self.call_count > self.max_calls:
+            raise UnexpectedCallReceived(
+                (
+                    "{}, {}: Unexpected call received.\n"
+                    "  Expected to receive at most {} calls, but an extra call was made."
+                ).format(_format_target(self.target), repr(self.method), self.max_calls)
+            )
 
     def add_accepted_args(self, *args, **kwargs):
         # TODO validate if args match callable signature
@@ -133,7 +141,13 @@ class _Runner(object):
         else:
             return "any arguments "
 
+    def _set_max_calls(self, times):
+        if not self.max_calls or (self.max_calls and times < self.max_calls):
+            self.max_calls = times
+
     def add_exact_calls_assertion(self, times):
+        self._set_max_calls(times)
+
         def assertion():
             if times != self.call_count:
                 raise AssertionError(
@@ -174,6 +188,8 @@ class _Runner(object):
         register_assertion(assertion)
 
     def add_at_most_calls_assertion(self, times):
+        self._set_max_calls(times)
+
         def assertion():
             if not self.call_count or self.call_count > times:
                 raise AssertionError(
@@ -438,7 +454,7 @@ class _MockCallableDSL(object):
         else:
             self._target = target
 
-        target_method_id = (id(target), method)
+        target_method_id = (id(self._target), method)
 
         if target_method_id not in self.CALLABLE_MOCKS:
             if not callable_mock:
@@ -639,11 +655,32 @@ class _MockCallableDSL(object):
     def and_assert_called_exactly(self, count):
         """
         Assert that there were exactly the given number of calls.
+
+        If assertion is for 0 calls, any received call will raise
+        UnexpectedCallReceived and also an AssertionError.
         """
+        if count is 0:
+            if self._runner:
+                raise ValueError(
+                    "Asked to not accept any calls, but a behavior was previously defined."
+                )
+            self.to_raise(
+                UnexpectedCallReceived(
+                    ("{}, {}: Excepted not to be called!").format(
+                        _format_target(self._target), repr(self._method)
+                    )
+                )
+            )
         self._assert_runner()
         self._runner.add_exact_calls_assertion(count)
         return self
 
+    def and_assert_not_called(self):
+        """
+        Short for and_assert_called_exactly(0)
+        """
+        return self.and_assert_called_exactly(0)
+
     def and_assert_called_once(self):
         """
         Short for and_assert_called_exactly(1)
@@ -681,11 +718,3 @@ class _MockCallableDSL(object):
         Short for self.and_assert_called_at_least(1).
         """
         return self.and_assert_called_at_least(1)
-
-    def and_assert_not_called(self):
-        """
-        Disallow calls, by raising UnexpectedCallReceived.
-        """
-        self._assert_runner()
-        self._runner.add_exact_calls_assertion(0)
-        return self
